class Pen
	attr_accessor :position_stack
	attr_reader :position
	attr_reader :heading
	attr_accessor :theta
	attr_accessor :speed #coefficent

	DEG2RAD = Math::PI/180

	#Targ can be either "gtk" or "cairo"... gtk calls the drawing area a drawing area
	#and cairo calls it a context, whilst gtk uses the context to switch the pen color
	#sweet confusing madness!
	def initialize( in_position, in_targ, in_da = nil, in_gc = nil )
		@position = in_position
		@targ = in_targ
		@da = in_da
		@gc = in_gc
		@up = false #starts out down
		@position_stack = []
		@heading = 90
		@theta = 60
		@unit_move = 10
		@speed = 1
		@reverse_coef = 1
		
		di_startup
	end
	
	#di = device independant
	def di_startup
		case @targ
			when "cairo"
				@da.move_to( *@position )
		end
	end
	
	def di_maybe_line( in_ox, in_oy, in_nx, in_ny )
		case @targ
			when "gtk"
				if( not is_up? )
					@da.window.draw_line( @gc, in_ox, in_oy, in_nx, in_ny ) 
				end
			when "cairo"
				if( not is_up? )
					@da.line_to( in_nx, in_ny )
				else
					@da.stroke
					@da.move_to( in_nx, in_ny )
				end
		end
	end
	
	def di_set_color( in_r, in_g, in_b )
		case @targ
			when "gtk"
				@gc.rgb_fg_color = Gdk::Color.new( in_r*256, in_g*256, in_b*256 )
			when "cairo"
				@da.stroke
				@da.move_to( *@position )
				@da.set_source_rgb( in_r/256.0, in_g/256.0, in_b/256.0 )
		end
	end

	def is_up?
		return @up
	end

	def pen_down
		@up = false
	end

	def pen_up
		@up = true
	end

	def position=(new_position)
		#p new_position
		old_position = @position
		@position = new_position
		di_maybe_line( old_position[0], old_position[1], new_position[0], new_position[1] ) 
	end

	def heading=(in_value)
		in_value = in_value.to_f
		while in_value < 0
			in_value += 360
		end

		@heading = (in_value % 360)
	end

	def forward( in_amount )
		if( in_amount )
			in_amount = in_amount.to_f
			dx = in_amount * Math.sin(@heading*DEG2RAD)
			dy = in_amount * Math.cos(@heading*DEG2RAD)
			self.position = [@position[0]+dx, @position[1]+dy]
		else
			pen_down
			dx = @unit_move * @speed * Math.sin(@heading*DEG2RAD)
			dy = @unit_move * @speed * Math.cos(@heading*DEG2RAD)
			self.position = [@position[0]+dx, @position[1]+dy]
		end
	end

	def back( in_amount )
		if( in_amount )
			in_amount = in_amount.to_f
			dx = in_amount * Math.sin(@heading*DEG2RAD)
			dy = in_amount * Math.cos(@heading*DEG2RAD)
			self.position = [@position[0]-dx, @position[1]-dy]
		else
			pen_down
			dx = @unit_move * @speed * Math.sin(@heading*DEG2RAD)
			dy = @unit_move * @speed * Math.cos(@heading*DEG2RAD)
			self.position = [@position[0]-dx, @position[1]-dy]
		end
	end

	def go
		pen_up
		dx = @unit_move * @speed * Math.sin(@heading*DEG2RAD)
		dy = @unit_move * @speed * Math.cos(@heading*DEG2RAD)
		self.position = [@position[0]+dx, @position[1]+dy]
	end

	def left
		@heading += @theta * @reverse_coef
	end

	def right
		@heading -= @theta * @reverse_coef
	end

	def about_face
		if( @heading >= 180 )
			@heading -= 180
		else
			@heading += 180
		end
	end

	def up_theta( in_amount )
		@theta += in_amount.to_f * @reverse_coef
	end

	def down_theta( in_amount )
		@theta -= in_amount.to_f * @reverse_coef
	end

	def push_position
		@position_stack.push( [@position, @heading] )
	end

	def pop_position
		pen_up
		self.position, @heading = @position_stack.pop
		pen_down
	end

	def go_reverse
		@reverse_coef = -@reverse_coef
	end

	def set_speed( in_inverse, in_sqrt, in_value )
		@speed = in_value.to_f
		@speed = 1 / @speed if in_inverse
		@speed = Math.sqrt( @speed ) if in_sqrt
	end

	def set_color( in_r, in_g, in_b )
		di_set_color( in_r, in_g, in_b )
	end

	def follow_instructions( in_instructions )
		scanner = StringScanner.new( in_instructions )
  		while inst = scanner.getch do
  			case inst
  				###
  				# Fractint compatibility
  				when "F", "D"
  					self.forward( scanner.get_real )
  				when "G", "M"
  					self.go
  				when "+"
  					self.left
  				when "-"
  					self.right
  				when "|"
  					self.about_face
  				when "\\"
  					the_value = scanner.get_real
  					self.up_theta( the_value )
  				when "/"
  					the_value = scanner.get_real
  					self.down_theta( the_value )
  				when "["
  					self.push_position
  				when "]"
  					self.pop_position
  				when "!"
  					self.go_reverse
  				when "@"
  					the_value = scanner.scan( /I?Q?[+-]?\d+\.?\d*/ )
  					self.set_speed( the_value[/I/], the_value[/Q/], the_value[/[+-]?\d+\.?\d*/] )
  				
  				###
  				# Turtle compatibility
  				when "^"
  					self.pen_up
  				when "#"
  					self.pen_down
  				when "S"
  					case scanner.getch
  						when "P"
		  					x = scanner.get_real
		  					scanner.getch
		  					y = scanner.get_real
		  					x = x.to_i + 250
		  					y = -y.to_i + 250
		  					self.position = [ x, y ]
		  				when "H"
		  					theta = scanner.get_real
		  					self.heading = theta
		  				when "T"
		  					self.theta = scanner.get_real.to_f
		  			end
		  		when "B"
		  			self.back( scanner.get_real )
		  		when "T"
		  			case scanner.getch
		  				when "R"
		  					theta = scanner.get_real
		  					theta = theta.to_f
		  					self.heading -= theta
		  				when "L"
		  					theta = scanner.get_real
		  					theta = theta.to_f
		  					self.heading += theta
		  			end
		  		when "C"
		  			r = scanner.get_real
		  			scanner.getch
		  			g = scanner.get_real
		  			scanner.getch
		  			b = scanner.get_real
					self.set_color( r.to_f, g.to_f, b.to_f )		  			
  			end
  		end
		return @da
	end
end
