require 'gtk2'
require 'strscan'
require 'renderers'

class StringScanner
	def get_real
		scan( /[+-]?\d+\.?\d*/ )
	end
end

require 'pen'
require 'turtle'

class BasicWindow < Gtk::Window
	def initialize
		@instructions = ""
		
		super(Gtk::Window::TOPLEVEL)
		set_title("Turtle Graphics")

		signal_connect("delete_event") do |widget, event|
			destroy
			Gtk.main_quit
			true
		end
		
		open_dialog = Gtk::FileChooserDialog.new("Open...", nil, 
						     Gtk::FileChooser::ACTION_OPEN,
						     "gnome-vfs",
						     [Gtk::Stock::OPEN, Gtk::Dialog::RESPONSE_ACCEPT],
						     [Gtk::Stock::CANCEL, Gtk::Dialog::RESPONSE_CANCEL]
				     )
		
		save_dialog = Gtk::FileChooserDialog.new("Save...", nil, 
						     Gtk::FileChooser::ACTION_SAVE,
						     "gnome-vfs",
						     [Gtk::Stock::SAVE, Gtk::Dialog::RESPONSE_ACCEPT],
						     [Gtk::Stock::CANCEL, Gtk::Dialog::RESPONSE_CANCEL]
				     )
		
		accel_group = Gtk::AccelGroup.new
		item_factory = Gtk::ItemFactory.new(Gtk::ItemFactory::TYPE_MENU_BAR, '<main>', accel_group)

		add_accel_group(accel_group)
		set_border_width(0)
		
		view = Gtk::TextView.new

		ifactory_cb = proc do |data, widget|
			the_choice = Gtk::ItemFactory.path_from_widget(widget)
			#puts "MenuItem: activated \"#{the_choice}\""
			case the_choice
				when /new$/i
					view.buffer.text = ""
				when /open$/i
					if open_dialog.run == Gtk::Dialog::RESPONSE_ACCEPT
						#puts "filename = #{open_dialog.filename}"
						view.buffer.text = IO.read( open_dialog.filename )
					end
					open_dialog.hide
				when /save$/i
					if save_dialog.run == Gtk::Dialog::RESPONSE_ACCEPT
						#puts "filename = #{save_dialog.filename}"
						File.open( save_dialog.filename, 'w' ) { |the_file| the_file.write( view.buffer.text ) }
					end
					save_dialog.hide
				when /render$/i
					puts "start rendering:"
					start_time = Time.now
					get_turtle_instructions( view )
					CairoRenderer.new( 500, 500, 'output.png', @instructions )
					puts "stop rendering, time: #{Time.now - start_time}"
				when /exit$/i
					destroy
					Gtk.main_quit
			end
		end

		menu_items = [
			['/_File'],
			['/File/_New', '<Item>', '<control>N', nil, ifactory_cb],
			['/File/_Open', '<Item>', '<control>O', nil, ifactory_cb],
			['/File/_Save', '<Item>', '<control>S', nil, ifactory_cb],
			['/File/sep1', '<Separator>'],
			['/File/_Render', '<Item>', '<control>R', nil, ifactory_cb],
			['/File/sep2', '<Separator>'],
			['/File/E_xit', '<Item>', nil, nil, ifactory_cb],
		]

		item_factory.create_items(menu_items)

		hbox = Gtk::HBox.new
		add(hbox)

		da = Gtk::DrawingArea.new
		# set a minimum size
		da.set_size_request(500, 500)

		hbox.add(da)
		
		da.signal_connect('expose_event') do |widget, event|
			redraw(widget)
		end
		
		#separator = Gtk::VSeparator.new
		#hbox.add(separator)

		vbox = Gtk::VBox.new
		hbox.add( vbox )

		vbox.pack_start(item_factory.get_widget('<main>'), false, false, 0)

		view.left_margin = 4
		view.set_size_request( 500, 500 )
		view.wrap_mode = Gtk::TextTag::WRAP_WORD
		
		vbox.add( view )
		
		button = Gtk::Button.new("Submit")
		button.signal_connect("clicked") {
			get_turtle_instructions( view )
			da.window.invalidate( Gdk::Region.new( Gdk::Rectangle.new( *da.window.geometry[0, 4] ) ), true ) 
		}
		vbox.add(button)
	end
	
	def get_turtle_instructions( in_view )
		the_turtle = Turtle.new
		begin
			the_turtle.instance_eval( in_view.buffer.text )
		rescue StandardError, SyntaxError
			puts "The code is effed up, heres what Ruby had to say about it:\n\n#{$!}"
		end
		@instructions = the_turtle.instructions
	end
	
	def redraw(da)
		gc = Gdk::GC.new(da.window)
		gc.set_line_attributes( 1, Gdk::GC::LINE_SOLID, Gdk::GC::CAP_ROUND, Gdk::GC::JOIN_ROUND)
		#gc.rgb_fg_color = Gdk::Color.new(rand(65535), rand(65535), rand(65535))

		gc.rgb_fg_color = Gdk::Color.new( 0, 0, 0 )
		
		the_pen = Pen.new( [250, 250], "gtk", da, gc )
		
		the_pen.follow_instructions( @instructions )
		
		return true
	end
end

window = BasicWindow.new
window.show_all
Gtk.main